"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from .client_context import ClientContext
from .function_context import FunctionContext

__all__ = ["ClientContext", "FunctionContext"]
